<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class provides PLESK support
 *
 * @package    OSB
 * @subpackage Plugins/Plesk
 * @category   Plugins
 * @author     Deon George
 * @copyright  (c) 2010 Deon George
 * @license    http://dev.leenooks.net/license.html
 */
abstract class Host_Plugin_Plesk extends Host_Plugin {
	// Service Object
	protected $protocol = '1.6.0.0';
	protected $path = 'enterprise/control/agent.php';
	protected $packet;
	protected $xml;
	private $_loaded;

	// Manage UI Login Attributes
	protected $url = 'login_up.php3';
	protected $login_acnt_field = '';
	protected $login_user_field = 'login_name';
	protected $login_pass_field = 'passwd';

	// Our required abstract classes
	public function admin_update() {
		return View::factory('host/admin/plugin/plesk')
			->set('o',$this);
	}

	public function manage_button(Model_Service_Plugin_Host $spho,$t) {
		return $this->render_button($t,$spho->service_id,$spho->username_value(),substr(md5($spho->password_value()),0,8));
	}
	public function admin_manage_button(Model_Host_Server $hso,$t) {
		return $this->render_button($t,$hso->id,substr(md5($hso->manage_username),0,8),substr(md5($hso->manage_password),0,8));
	}

	protected function render_button($t,$sid,$u,$p) {
		$debug = FALSE;
		$output = '';

		$output .= Form::open(
			$debug ? 'debug/site' : sprintf('%s/%s',$this->hso->manage_url,$this->url),
			array('target'=>'w24','method'=>'post','id'=>sprintf('id_%s_%s',$sid,$t))
		);
		$output .= Form::input($this->login_user_field,$u,array('type'=>'hidden','id'=>sprintf('u_%s_%s',$sid,$t)));
		$output .= Form::input($this->login_pass_field,$p,array('type'=>'hidden','id'=>sprintf('p_%s_%s',$sid,$t)));
		$output .= Form::close();
		$output .= Form::button('submit',_('Manage'),array('class'=>'form_button','value'=>sprintf('%s:%s',$sid,$t)));

		return $output;
	}

	protected function init() {
		$this->_loaded = FALSE;
		$this->xml = XML::factory(NULL,'plesk');
		$this->packet = $this->xml->add_node('packet','',array('version'=>$this->protocol));
	}

	protected function server_command(XML $xml) {
		// We need to find out the command key, so we can get the status result.
		if (count($a=array_keys($xml->packet->as_array())) != 1)
			throw new Kohana_Exception('XML command malformed? :xml',array(':xml'=>(string)$xml));
		$key = array_shift($a);
		if (count($a=array_keys($xml->packet->$key->as_array())) != 1)
			throw new Kohana_Exception('XML command malformed? :xml',array(':xml'=>(string)$xml));
		$get = array_shift($a);

		$request = Request::factory(sprintf('%s/%s',$this->hso->manage_url,$this->path))
			->method('POST');

		$request->get_client()->options(Arr::merge($this->curlopts,array(
			CURLOPT_HTTPHEADER => array(
				'HTTP_AUTH_LOGIN: '.$this->hso->manage_username,
				'HTTP_AUTH_PASSWD: '.$this->hso->manage_password,
				'Content-Type: text/xml',
			),
			CURLOPT_POSTFIELDS => $this->render($xml),
			)));

		$response = $request->execute();

		$result = XML::factory(NULL,'plesk',$response->body());

		if ($result->status->value() == 'ok')
			$this->_loaded = TRUE;

		return $result;
	}

	protected function collapse(XML $xml) {
		$result = array();

		foreach ($xml->as_array() as $j=>$k) {
			$v = $xml->$j->value();

			if (count($k) > 1) {
				foreach ($xml->get($j,1) as $k)
					if (isset($k['name'][0]))
						$result[$j][$k['name'][0]] = (isset($k['value'][0]) ? $k['value'][0] : '');
					else
						$result[$j][] = $k;

			} elseif (! is_null($v))
				$result[$j] = $v;
			else
				$result[$j] = $this->collapse($xml->$j);
		}

		if (array_key_exists('name',$result) AND array_key_exists('value',$result))
			$result = array($result['name']=>$result['value']);

		return $result;
	}

	public function loaded() {
		return $this->_loaded;
	}

	private function render(XML $xml) {
		return preg_replace('/<\/?plesk>/','',(string)$xml->render(FALSE));
	}
}
?>
